﻿using System;
using Neuron.Esb;
using System.Xml.Serialization;

namespace Neuron.Esb.Sample
{
    [XmlRoot("TestRoot"), Serializable]  
    public class TestSerialization
    {
        [XmlElement("Name")]  
        public string Name { get; set; }
        [XmlElement("Age")]  
        public int Age { get; set; }
        [XmlElement("Sex")]  
        public string Sex { get; set; }
    }
    class Program
    {
        /// <summary>
        /// set the topic to publish to and the name of the publisher
        /// </summary>
        const string TOPIC = "Orders";
        const string PUBLISHER = "OrderEntry";

        static void Main(string[] args)
        {

            DateTime dateTimeObject = DateTime.Now;
            TestSerialization testObject = new TestSerialization() { Age = 5, Name = "Joe", Sex = "?" };

            /// <summary>
            /// There are two ways to Connect a Neuron Party  Either during "Connect()" we can automatically connect using the settings from the app.config file
            /// Or...we can specifically provide that information by passing it in the Publisher constructor using the SubscriberConfiguration class .
            /// The first example looks like this:
            /// 
            ///     using (Publisher publisher = new Publisher(PUBLISHER))
            /// The second example would look like this:
            /// 
            ///     using (Publisher publisher = new Publisher(new SubscriberConfiguration() { Zone = "Enterprise", ServiceAddress = "net.tcp://localhost:50000", SubscriberId = PUBLISHER }))
            /// </summary>
                
            // Create an instance of a publisher
            using (Publisher publisher = new Publisher(PUBLISHER))
            {
                publisher.OnOnline += new Channels.TopicEventHandler(publisher_OnOnline);

                // catch any exceptions that may occurr while connecting to each individual topic
                PartyConnectExceptions exceptions = publisher.Connect();
                if (exceptions != null && exceptions.Count > 0)
                {
                    foreach (var e in exceptions.GetResults())
                        Console.WriteLine(string.Format("An error occurred connecting '{0}' to Topic, '{1}'. {2}", publisher.Context.PartyId, e.Exception.Message, e.Topic));
                }
                else
                {

                    #region Send some XML to the bus
                    // Send some XML.  Body type is automatically set
                    publisher.SendXml(TOPIC, "<Test>dsfdsf</Test>");
                    #endregion

                    #region Send an ESBMessage type
                    // send an ESB Message with custom properties
                    ESBMessage msg = new ESBMessage(TOPIC, "hi mom");
                    msg.SetProperty("emerson", "po", "98ijk");
                    msg.Header.BodyType = "text/xml";
                    
                    publisher.SendMessage(msg);
                    #endregion

                    #region Send a .NET serializable object like datetime
                    // Send anything .net serializable like date time or custom class
                    publisher.Send(TOPIC, DateTime.Now);
                    #endregion

                    #region Send a .NET serializable custom class
                    publisher.Send(TOPIC, testObject);
                    #endregion

                    #region Send a simple request message that will return a reply
                    // this means that the subscriber must send back a reply message
                    Console.WriteLine("<Test>My Request</Test>");
                    ESBMessage response = publisher.SendXml(TOPIC, "<Test>My Request</Test>","",SendOptions.Request);

                    // print out the message
                    PrintOutMessageByType(response);
                    #endregion
                }

                Console.ReadLine();
            }
        }

        private static void publisher_OnOnline(object o, TopicEventArgs e)
        {
            if(e != null) Console.WriteLine("  Online Event = " + e.Topic);
        }

        private static void PrintOutMessageByType(ESBMessage message)
        {
            if (message.Header.BodyType != null)
            {
                if (message.Header.BodyType.Equals("text/xml"))
                    Console.WriteLine(message.GetBody<string>());

                if (message.Header.BodyType.Equals("DateTime"))
                    Console.WriteLine(message.GetBody<DateTime>().ToString());

                if (message.Header.BodyType.Equals("TestSerialization"))
                    Console.WriteLine(message.GetBody<TestSerialization>().Name);
            }
            else
            {
                Console.WriteLine(message.Text);
            }
        }































        //private static void PrintOutMessageByType(ESBMessage message)
        //{
        //    // first print out the message
        //    if (message.Header.BodyType != null)
        //    {
        //        if (message.Header.BodyType.Equals("text/xml"))
        //        {
        //            string objString = message.GetBody<string>();
        //            Console.WriteLine(objString.ToString());
        //        }

        //        if (message.Header.BodyType.Equals("DateTime"))
        //        {
        //            DateTime objDateTime = message.GetBody<DateTime>();
        //            Console.WriteLine(objDateTime.ToString());
        //        }
        //    }
        //    else
        //    {
        //        Console.WriteLine(message.Text);
        //    }
        //}

        //private static void OnReceive(object sender, MessageEventArgs e)
        //{
        //    ESBMessage message = e.Message;

        //    // print out the message
        //    PrintOutMessageByType(message);

        //}
    }
}
